@@ -30,6 +30,7 @@
 #include <profiles/gnome-media-profiles.h>
 #include <nautilus-burn-drive.h>
 #include <nautilus-burn-drive-selection.h>
+#include "gconf-bridge.h"
 #include "sj-extracting.h"
 
 extern GladeXML *glade;
@@ -39,7 +40,6 @@ static GtkWidget *audio_profile;
 static GtkWidget *prefs_dialog;
 static GtkWidget *cd_option, *path_option, *file_option, *basepath_fcb, *check_strip, *check_eject;
 static GtkWidget *path_example_label;
-static GList *cdroms = NULL;
 
 #define DEFAULT_AUDIO_PROFILE_NAME "cdlossy"
 
@@ -50,10 +50,14 @@ typedef struct {
 
 static const FilePattern path_patterns[] = {
   {N_("Album Artist, Album Title"), "%aa/%at"},
+  {N_("Album Artist (sortable), Album Title"), "%as/%at"},
   {N_("Track Artist, Album Title"), "%ta/%at"},
+  {N_("Track Artist (sortable), Album Title"), "%ts/%at"},
   {N_("Album Title"), "%at"},
   {N_("Album Artist"), "%aa"},
+  {N_("Album Artist (sortable)"), "%as"},
   {N_("Album Artist - Album Title"), "%aa - %at"},
+  {N_("Album Artist (sortable) - Album Title"), "%as - %at"},
   {N_("[none]"), ""},
   {NULL, NULL}
 };
@@ -62,56 +66,14 @@ static const FilePattern file_patterns[] = {
   {N_("Number - Title"), "%tN - %tt"},
   {N_("Track Title"), "%tt"},
   {N_("Track Artist - Track Title"), "%ta - %tt"},
+  {N_("Track Artist (sortable) - Track Title"), "%ts - %tt"},
   {N_("Number. Track Artist - Track Title"), "%tN. %ta - %tt"},
+  /* {N_("Number. Track Artist (sortable) - Track Title"), "%tN. %ts - %tt"}, */
   {N_("Number-Track Artist-Track Title (lowercase)"), "%tN-%tA-%tT"},
+  /* {N_("Number-Track Artist (sortable)-Track Title (lowercase)"), "%tN-%tS-%tT"}, */
   {NULL, NULL}
 };
 
-const char* prefs_get_default_device ()
-{
-  static const char* default_device = NULL;
-  if (default_device == NULL) {
-    NautilusBurnDrive *cd;
-    cdroms = nautilus_burn_drive_get_list (FALSE, FALSE);
-    if (cdroms == NULL) return NULL;
-    cd = cdroms->data;
-    default_device = cd->device;
-  }
-  return default_device;
-}
-
-gboolean cd_drive_exists (const char *device)
-{
-  GList *l;
-  if (device == NULL) return FALSE;
-  if (cdroms == NULL) {
-    cdroms = nautilus_burn_drive_get_list (FALSE, FALSE);
-  }
-  for (l = cdroms; l != NULL; l = l->next) {
-    if (strcmp (((NautilusBurnDrive *) (l->data))->device, device) == 0)
-      return TRUE;
-  }
-  return FALSE;
-}
-
-/**
- * Changed the CD-ROM device in the prefs dialog
- */
-void prefs_cdrom_changed_cb (GtkWidget *widget, const char* device)
-{
-  gconf_client_set_string (gconf_client, GCONF_DEVICE, device != NULL ? device : "", NULL);
-}
-
-/**
- * The Eject when Finished check was toggled.
- */
-void prefs_eject_toggled (GtkToggleButton *togglebutton, gpointer user_data)
-{
-  gconf_client_set_bool (gconf_client, GCONF_EJECT,
-                         gtk_toggle_button_get_active (togglebutton),
-                         NULL);
-}
-
 void prefs_profile_changed (GtkWidget *widget, gpointer user_data)
 {
   GMAudioProfile *profile;
@@ -182,16 +144,6 @@ void prefs_file_option_changed (GtkComboBox *combo, gpointer user_data)
   }
 }
 
-/**
- * The Strip Special Characters check was toggled.
- */
-void prefs_strip_toggled (GtkToggleButton *togglebutton, gpointer user_data)
-{
-  gconf_client_set_bool (gconf_client, GCONF_STRIP,
-                         gtk_toggle_button_get_active (togglebutton),
-                         NULL);
-}
-
 /**
  * The Edit Profiles button was pressed.
  */
@@ -203,20 +155,6 @@ void prefs_edit_profile_clicked (GtkButton *button, gpointer user_data)
   gtk_dialog_run (GTK_DIALOG (dialog));
 }
 
-static void device_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
-{
-  g_return_if_fail (strcmp (entry->key, GCONF_DEVICE) == 0);
-
-  if (entry->value == NULL) {
-    nautilus_burn_drive_selection_set_device (NAUTILUS_BURN_DRIVE_SELECTION (cd_option),
-                                   nautilus_burn_drive_selection_get_default_device (NAUTILUS_BURN_DRIVE_SELECTION (cd_option)));
-  } else {
-    g_return_if_fail (entry->value->type == GCONF_VALUE_STRING);
-    nautilus_burn_drive_selection_set_device (NAUTILUS_BURN_DRIVE_SELECTION (cd_option),
-                                   gconf_value_get_string (entry->value));
-  }
-}
-
 static void audio_profile_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   const char *value;
@@ -241,7 +179,7 @@ static void baseuri_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *
    * prefs window from the main is ugly. Maybe put the validation code
    * into sj-utils?
    */
-  const char* base_uri;
+  const char* base_uri, *current_uri;
   g_return_if_fail (strcmp (entry->key, GCONF_BASEURI) == 0);
 
   if (entry->value == NULL) {
@@ -249,70 +187,61 @@ static void baseuri_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *
   } else {
     g_return_if_fail (entry->value->type == GCONF_VALUE_STRING);
     base_uri = gconf_value_get_string (entry->value);
-    if (strcmp (gtk_file_chooser_get_current_folder_uri (GTK_FILE_CHOOSER (basepath_fcb)), base_uri) != 0) { 
+    current_uri = gtk_file_chooser_get_current_folder_uri (GTK_FILE_CHOOSER (basepath_fcb));
+    if (current_uri == NULL || strcmp (current_uri, base_uri) != 0) { 
       gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (basepath_fcb), base_uri);
     }
   }
 }
 
-static void strip_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
-{
-  gboolean value = FALSE;
-  g_return_if_fail (strcmp (entry->key, GCONF_STRIP) == 0);
-
-  if (entry->value != NULL) {
-    g_return_if_fail (entry->value->type == GCONF_VALUE_BOOL);
-    value = gconf_value_get_bool (entry->value);
-  }
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_strip), value);
-}
-
-static void eject_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
-{
-  gboolean value = FALSE;
-  g_return_if_fail (strcmp (entry->key, GCONF_EJECT) == 0);
-
-  if (entry->value != NULL) {
-    g_return_if_fail (entry->value->type == GCONF_VALUE_BOOL);
-    value = gconf_value_get_bool (entry->value);
-  }
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_eject), value);
-}
-
 static void pattern_label_update (void)
 {
   char *file_pattern, *path_pattern;
   char *file_value, *path_value, *example, *format;
-  gboolean strip;
+  GMAudioProfile *profile;
+
   static AlbumDetails sample_album = {
-    N_("Album Title"), /* title */
-    N_("Album Artist"), /* artist */
+    /* Translators: this is the title of a common album. You don't need to change it */
+    N_("Help!"), /* title */
+    /* Translators: this is the artist of a common album. You don't need to change it */
+    N_("The Beatles"), /* artist */
+    /* Translators: this is the artist sortname of a common album. You don't need to change it */
+    N_("Beatles, The"), /* sortname */
+    NULL, /* genre */
     0, /* number of tracks*/
     NULL, /* track list */
     NULL, /* release date */
     NULL, /* album ID */
-    NULL, /* artist ID */
+    NULL /* artist ID */
   };
   static TrackDetails sample_track = {
     &sample_album,  /*album */
-    9, /* track number */
-    N_("Track Title"), /* title */
-    N_("Track Artist"), /* artist */
-    60, /* duration */
-    LAST_GENRE, /* genre */
+    7, /* track number */
+    /* Translators: this is the title of a common song. You don't need to change it */
+    N_("Ticket To Ride"), /* title */
+    /* Translators: this is the artist of a common song. You don't need to change it */
+    N_("The Beatles"), /* artist */
+    /* Translators: this is the artist sortname of a common song. You don't need to change it */
+    N_("Beatles, The"), /* sortname */
+    0, /* duration */
     NULL, /* track ID */
     NULL, /* artist ID */
-    {0,0,0,0} /* treeview iterator */
+    {0,0,0,0}, /* treeview iterator */
   };
   /* Magic to i18n-ize the sample strings. */
   static gboolean been_here = FALSE;
   if (!been_here) {
     sample_album.title = _(sample_album.title);
     sample_album.artist = _(sample_album.artist);
+    sample_album.artist_sortname = _(sample_album.artist_sortname);
     sample_track.title = _(sample_track.title);
     sample_track.artist = _(sample_track.artist);
+    sample_track.artist_sortname = _(sample_track.artist_sortname);
     been_here = TRUE;
   }
+  
+  g_object_get (extractor, "profile", &profile, NULL);
+  
   /* TODO: sucky. Replace with get-gconf-key-with-default mojo */
   file_pattern = gconf_client_get_string (gconf_client, GCONF_FILE_PATTERN, NULL);
   if (file_pattern == NULL) {
@@ -322,7 +251,6 @@ static void pattern_label_update (void)
   if (path_pattern == NULL) {
     path_pattern = g_strdup(path_patterns[0].pattern);
   }
-  strip = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check_strip));
 
   file_value = filepath_parse_pattern (file_pattern, &sample_track);
   path_value = filepath_parse_pattern (path_pattern, &sample_track);
@@ -333,7 +261,11 @@ static void pattern_label_update (void)
   g_free (path_value);
   g_free (path_pattern);
 
-  format = g_strconcat ("<i><small>", example, "</small></i>", NULL);
+  format = g_strconcat ("<small><i><b>Example Path:</b> ",
+                        example,
+                        ".",
+                        gm_audio_profile_get_extension(profile),
+                        "</i></small>", NULL);
   g_free (example);
   
   gtk_label_set_markup (GTK_LABEL (path_example_label), format);
@@ -406,10 +338,26 @@ void on_edit_preferences_cb (GtkMenuItem *item, gpointer user_data)
   int rc;
 
   if (prefs_dialog == NULL) {
+    const char * labels[] = { "cd_label", "path_label", "folder_label", "file_label", "profile_label" };
+    guint i;
+    GtkSizeGroup *group;
+    GConfBridge *bridge = gconf_bridge_get ();
+
     prefs_dialog = glade_xml_get_widget (glade, "prefs_dialog");
     g_assert (prefs_dialog != NULL);
     gtk_window_set_transient_for (GTK_WINDOW (prefs_dialog), GTK_WINDOW (main_window));
 
+    group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+    for (i = 0; i < G_N_ELEMENTS (labels); i++) {
+      GtkWidget *widget;
+      widget = glade_xml_get_widget (glade, labels[i]);
+      if (widget) {
+        gtk_size_group_add_widget (group, widget);
+      } else {
+        g_warning ("Widget %s not found", labels[i]);
+      }
+    }
+    g_object_unref (group);
 
     cd_option = glade_xml_get_widget (glade, "cd_option");
     basepath_fcb = glade_xml_get_widget (glade, "path_chooser");
@@ -426,19 +374,18 @@ void on_edit_preferences_cb (GtkMenuItem *item, gpointer user_data)
     g_signal_connect (file_option, "changed", G_CALLBACK (prefs_file_option_changed), NULL);
 
     /* Connect to GConf to update the UI */
-    gconf_client_notify_add (gconf_client, GCONF_DEVICE, device_changed_cb, NULL, NULL, NULL);
+    gconf_bridge_bind_property (bridge, GCONF_EJECT, G_OBJECT (check_eject), "active");
+    gconf_bridge_bind_property (bridge, GCONF_STRIP, G_OBJECT (check_strip), "active");
+    gconf_bridge_bind_property (bridge, GCONF_DEVICE, G_OBJECT (cd_option), "device");
     gconf_client_notify_add (gconf_client, GCONF_BASEURI, baseuri_changed_cb, NULL, NULL, NULL);
     gconf_client_notify_add (gconf_client, GCONF_AUDIO_PROFILE, audio_profile_changed_cb, NULL, NULL, NULL);
-    gconf_client_notify_add (gconf_client, GCONF_STRIP, strip_changed_cb, NULL, NULL, NULL);
-    gconf_client_notify_add (gconf_client, GCONF_EJECT, eject_changed_cb, NULL, NULL, NULL);
     gconf_client_notify_add (gconf_client, GCONF_PATH_PATTERN, path_pattern_changed_cb, NULL, NULL, NULL);
     gconf_client_notify_add (gconf_client, GCONF_FILE_PATTERN, file_pattern_changed_cb, NULL, NULL, NULL);
+
+    g_signal_connect (extractor, "notify::profile", pattern_label_update, NULL);
   }
   baseuri_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_BASEURI, NULL, TRUE, NULL), NULL);
-  device_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_DEVICE, NULL, TRUE, NULL), NULL);
   audio_profile_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_AUDIO_PROFILE, NULL, TRUE, NULL), NULL);
-  strip_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_STRIP, NULL, TRUE, NULL), NULL);
-  eject_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_EJECT, NULL, TRUE, NULL), NULL);
   file_pattern_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_FILE_PATTERN, NULL, TRUE, NULL), NULL);
   path_pattern_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_PATH_PATTERN, NULL, TRUE, NULL), NULL);
 
